USE [BigFactTable]
GO

-- Row count = 123,695,104
SELECT  COUNT(*) AS [rowcount]
FROM    dbo.FactInternetSales;
GO

-- ** Turn on ACTUAL plan (which I'll probably forget to do anyhow) **
-- Takes 55 seconds to execute
SELECT  p.ProductLine,
        SUM(f.SalesAmount) TotalSalesAmount
FROM    [dbo].[FactInternetSales] f
INNER JOIN [dbo].[DimProduct] p
        ON f.ProductKey = p.ProductKey
GROUP BY p.ProductLine
ORDER BY p.ProductLine
OPTION  (IGNORE_NONCLUSTERED_COLUMNSTORE_INDEX);


-- Takes about 7 minutes to create
CREATE NONCLUSTERED COLUMNSTORE INDEX [NCI_FactInternetSales] 
ON [dbo].[FactInternetSales]
(
	[ProductKey],
	[OrderDateKey],
	[DueDateKey],
	[ShipDateKey],
	[CustomerKey],
	[PromotionKey],
	[CurrencyKey],
	[SalesTerritoryKey],
	[SalesOrderNumber],
	[SalesOrderLineNumber],
	[RevisionNumber],
	[OrderQuantity],
	[UnitPrice],
	[ExtendedAmount],
	[UnitPriceDiscountPct],
	[DiscountAmount],
	[ProductStandardCost],
	[TotalProductCost],
	[SalesAmount],
	[TaxAmt],
	[Freight],
	[CarrierTrackingNumber],
	[CustomerPONumber],
	[OrderDate],
	[DueDate],
	[ShipDate]
)WITH (DROP_EXISTING = OFF)
GO

-- Turn on ACTUAL plan
-- Should run in < 1 second
SELECT  p.ProductLine,
        SUM(f.SalesAmount) TotalSalesAmount
FROM    [dbo].[FactInternetSales] f
INNER JOIN [dbo].[DimProduct] p
        ON f.ProductKey = p.ProductKey
GROUP BY p.ProductLine
ORDER BY p.ProductLine;

-- What indexes now exist for this table?
EXEC sp_helpindex 'FactInternetSales';
GO


-- 27 columns (includes uniqueifier) * 130 segments = 3510 rows
SELECT  [partition_id],
        [hobt_id],
        [column_id],
        [segment_id],
        [version],
        [encoding_type],
        [row_count],
        [has_nulls],
        [base_id],
        [magnitude],
        [primary_dictionary_id],
        [secondary_dictionary_id],
        [min_data_id],
        [max_data_id],
        [null_value],
        [on_disk_size]
FROM    sys.column_store_segments;

-- Used page allocation
SELECT  [partition_id],
        [object_id],
        [index_id],
        [partition_number],
        [in_row_used_page_count],
        [lob_used_page_count],
        [row_overflow_used_page_count],
        [used_page_count],
        [row_count]
FROM    sys.dm_db_partition_stats
WHERE   [partition_id] = 72057594042712064;


-- Columnstore meta-data
SELECT  [partition_id],
        [hobt_id],
        [column_id],
        [dictionary_id],
        [version],
        [type],
        [flags],
        [last_id],
        [entry_count],
        [on_disk_size]
FROM    sys.column_store_dictionaries;

