-- Validating row vs. batch mode

/*
 SELECT ProductLine FROM [dbo].[DimProduct] ;
*/

USE [BigFactTable]
GO

-- Row execution mode
SELECT  p.ProductKey,
        ISNULL(SUM(f.SalesAmount),0.00) AS [TotalSalesAmount]
FROM    [dbo].[DimProduct] AS p
LEFT OUTER JOIN [dbo].[FactInternetSales] AS f
        ON f.ProductKey = p.ProductKey
GROUP BY p.ProductKey
ORDER BY p.ProductKey;
GO

-- Batch execution mode alternative

WITH    IntermediateResult
          AS (SELECT    p.ProductKey,
                        SUM(f.SalesAmount) AS [TotalSalesAmount]
              FROM      [dbo].[DimProduct] AS p
              INNER JOIN [dbo].[FactInternetSales] AS f
                        ON f.ProductKey = p.ProductKey
              GROUP BY  p.ProductKey)
SELECT  p.[ProductKey],
        ISNULL(i.[TotalSalesAmount], 0) AS [TotalSalesAmount]
FROM    [dbo].[DimProduct] AS p
LEFT OUTER JOIN IntermediateResult AS i
        ON p.[ProductKey] = i.[ProductKey]
ORDER BY p.[ProductKey];

 

-- Scalar aggregate - row mode
SELECT  SUM(f.SalesAmount) AS [TotalSalesAmount]
FROM    [dbo].[FactInternetSales] AS f
GO

-- Scalar aggregate - batch mode
WITH IntermediateResult 
AS (
	SELECT  YEAR([OrderDate]) AS [OrderDateYear], 
			SUM(f.SalesAmount) AS [TotalSalesAmount]
	FROM    [dbo].[FactInternetSales] AS f
	GROUP BY YEAR([OrderDate]) 
 ) 
SELECT SUM([TotalSalesAmount])
FROM IntermediateResult;
GO

