-- Data movement techniques

-- Switch OUT to a staging table

USE [BigFactTable]
GO


CREATE TABLE [dbo].[staging_FactInternetSales](
	[ProductKey] [int] NOT NULL,
	[OrderDateKey] [int] NOT NULL,
	[DueDateKey] [int] NOT NULL,
	[ShipDateKey] [int] NOT NULL,
	[CustomerKey] [int] NOT NULL,
	[PromotionKey] [int] NOT NULL,
	[CurrencyKey] [int] NOT NULL,
	[SalesTerritoryKey] [int] NOT NULL,
	[SalesOrderNumber] [nvarchar](20) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[SalesOrderLineNumber] [tinyint] NOT NULL,
	[RevisionNumber] [tinyint] NOT NULL,
	[OrderQuantity] [smallint] NOT NULL,
	[UnitPrice] [money] NOT NULL,
	[ExtendedAmount] [money] NOT NULL,
	[UnitPriceDiscountPct] [float](53) NOT NULL,
	[DiscountAmount] [float](53) NOT NULL,
	[ProductStandardCost] [money] NOT NULL,
	[TotalProductCost] [money] NOT NULL,
	[SalesAmount] [money] NOT NULL,
	[TaxAmt] [money] NOT NULL,
	[Freight] [money] NOT NULL,
	[CarrierTrackingNumber] [nvarchar](25) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[CustomerPONumber] [nvarchar](25) COLLATE SQL_Latin1_General_CP1_CI_AS NULL,
	[OrderDate] [datetime] NULL,
	[DueDate] [datetime] NULL,
	[ShipDate] [datetime] NULL
) ON [PRIMARY]
WITH
(
DATA_COMPRESSION = PAGE
)
GO

CREATE NONCLUSTERED COLUMNSTORE INDEX [staging_FactInternetSales_NCI_FactInternetSales] ON [dbo].[staging_FactInternetSales]
(
	[ProductKey],
	[OrderDateKey],
	[DueDateKey],
	[ShipDateKey],
	[CustomerKey],
	[PromotionKey],
	[CurrencyKey],
	[SalesTerritoryKey],
	[SalesOrderNumber],
	[SalesOrderLineNumber],
	[RevisionNumber],
	[OrderQuantity],
	[UnitPrice],
	[ExtendedAmount],
	[UnitPriceDiscountPct],
	[DiscountAmount],
	[ProductStandardCost],
	[TotalProductCost],
	[SalesAmount],
	[TaxAmt],
	[Freight],
	[CarrierTrackingNumber],
	[CustomerPONumber],
	[OrderDate],
	[DueDate],
	[ShipDate]
)WITH (DROP_EXISTING = OFF) ON [PRIMARY]
GO


CREATE CLUSTERED INDEX [staging_FactInternetSales_NCI_FactInternetSales_ProductKey] ON [dbo].[staging_FactInternetSales]
(
	[ProductKey] ASC,
	[OrderDate]
)WITH (PAD_INDEX = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
GO

ALTER TABLE [dbo].[staging_FactInternetSales]  WITH CHECK ADD  CONSTRAINT [chk_staging_FactInternetSales_partition_2] CHECK  ([OrderDate]>=N'2001-07-01T00:00:00' AND [OrderDate]<N'2002-07-01T00:00:00' AND [OrderDate] IS NOT NULL)
ALTER TABLE [dbo].[staging_FactInternetSales] CHECK CONSTRAINT [chk_staging_FactInternetSales_partition_2]
GO

-- *****
-- Our actual switch operation
-- *****

ALTER TABLE [BigFactTable].[dbo].[FactInternetSales] 
SWITCH PARTITION 2 TO 
[BigFactTable].[dbo].[staging_FactInternetSales]
GO

-- Validate data
SELECT COUNT(*) 
FROM [dbo].[staging_FactInternetSales];

-- What about the source table?  Can I perform an INSERT?
INSERT  [BigFactTable].[dbo].[FactInternetSales]
        SELECT  ProductKey,
                OrderDateKey,
                DueDateKey,
                ShipDateKey,
                CustomerKey,
                PromotionKey,
                CurrencyKey,
                SalesTerritoryKey,
                SalesOrderNumber,
                SalesOrderLineNumber,
                RevisionNumber,
                OrderQuantity,
                UnitPrice,
                ExtendedAmount,
                UnitPriceDiscountPct,
                DiscountAmount,
                ProductStandardCost,
                TotalProductCost,
                SalesAmount,
                TaxAmt,
                Freight,
                CarrierTrackingNumber,
                CustomerPONumber,
                OrderDate,
                DueDate,
                ShipDate
        FROM    [dbo].[staging_FactInternetSales];

-- Switch IN
ALTER TABLE [BigFactTable].[dbo].[staging_FactInternetSales] 
SWITCH  TO [BigFactTable].[dbo].[FactInternetSales]PARTITION 2


-- Validate data
SELECT COUNT(*) 
FROM [dbo].[staging_FactInternetSales];


-- Validate data
SELECT COUNT(*) 
FROM [dbo].[FactInternetSales];


-- Validate data

-- Cleanup
DROP TABLE [dbo].[staging_FactInternetSales];
GO
